function brick(x,y,z,sx,sy,sz,col)
%BRICK draw a 3D brick 
%brick(x,y,z,sx,sy,sz,col)
%
% x,y,z: location of the brick
% sz,sy,sz: size of the brick
% col: (optional) color. 


% This file is part of the Buildable Evolution Software Project. 
% 
% http://www.demo.cs.brandeis.edu/pr/buildable
% (C) Pablo Funes and Jordan B. Pollack 1997-2008
% pablo@cs.brandeis.edu
% 
% The methods and configurations incorporated in the Builable Evolution
% Software Project are covered in part by United States patent
% 6,434,492 "Computer apparatus and method for analyzing structural
% stability" of August 13, 2002. 
% 
% The Buildable Evolution Software Project is free software: you 
% can redistribute it and/or modify it under the terms of the 
% GNU General Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your option) 
% any later version. 
% 
% The Buildable Evolution Software Project is distributed in the
% hope that it will be useful, but WITHOUT ANY WARRANTY; without
% even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more
% details. 
% 
% You should have received a copy of the GNU General Public License 
% along with the software. If not, see <http://www.gnu.org/licenses/>. 
fprintf('.');
solid=true;
lid=false;
knobs=true;
usecolormap=true;
lw=1; % linewidth
fl=round((z+sz)/1.195);
%global floor
%if (length(floor) > 1 && (fl < floor(1) || fl > floor(2)))
%    return
%end
gap = 0.04; % gap between bricks;
kr = 0.225; % knob radius
kh = 0.12; % knob height
z=z + gap/2;
sz= sz - gap/2;
%z=z+0.025;
%sz=sz-0.05;
if nargin< 7
    usercolor = 0;
    if ~usecolormap
        %colors=['m','c','k',];
        %col=colors(1+mod(fl,length(colors)));
        cmap=colormap;
        col=cmap(1+floor(size(cmap,1)*(fl/45+0.5)),:);
        usercolor = 1;
    else
        %cmap=colormap;
        %col=cmap(fl+1,:);
        col = fl;
        usercolor = 0;
    end
else
    usercolor=1;
end
hold on
if ~solid
    % wireframe
    x1=[0 0 0 0 0 1 1 1 1 1]*sx+x;
    y1=[0 1 1 0 0 0 1 1 0 0]*sy+y;
    z1=[0 0 1 1 0 0 0 1 1 0]*sz+z;
    xa=[ 0 1]*sx+x;
    ya=[ 0 0]*sy+y;
    za=[ 1 1]*sz+z;
    xb=[0 1]*sx+x;
    yb=[1 1]*sy+y;
    zb=[1 1]*sz+z;
    xc=[0 1]*sx+x;
    yc=[1 1]*sy+y;
    zc=[0 0]*sz+z;
    plot3(x1,y1,z1,'color',col,'linewidth',lw)
    plot3(xa,ya,za,'color',col,'linewidth',lw)
    plot3(xb,yb,zb,'color',col,'linewidth',lw)
    plot3(xc,yb,zc,'color',col,'linewidth',lw)
    if lid
        % paint a lid on top of the bricks
        topx=[0 1 1 0]*(sx-0.2)+x+0.1;
        topy=[0 0 1 1]*(sy-0.2)+y+0.1;
        topz=[1 1 1 1]*sz+z;
        fill3(topx,topy,topz,col)
    end
else
    gx = x + gap/2;
    gy = y + gap/2;
    gsx = sx - gap/2;
    gsy = sy - gap/2;
    patch([0 0 1 1]*gsx+gx,[0 0 0 0]*gsy+gy,[0 1 1 0]*sz+z,col);
    patch([0 0 1 1]*gsx+gx,[1 1 1 1]*gsy+gy,[0 1 1 0]*sz+z,col);
    patch([0 0 1 1]*gsx+gx,[0 1 1 0]*gsy+gy,[0 0 0 0]*sz+z,col);
    patch([0 0 1 1]*gsx+gx,[0 1 1 0]*gsy+gy,[1 1 1 1]*sz+z,col);
    patch([0 0 0 0]*gsx+gx,[0 0 1 1]*gsy+gy,[0 1 1 0]*sz+z,col);
    patch([1 1 1 1]*gsx+gx,[0 0 1 1]*gsy+gy,[0 1 1 0]*sz+z,col);
end
if knobs
    [cx,cy,cz]=cylinder([1 1],9);
    cx = cx * kr;
    cy = cy * kr; % knob radius
    cz = cz * kh; % knob height
    if ~usercolor
        ccol = col * ones(size(cz));
    end
    for i=1:sx
        for j=1:sy
            tx = x+i-0.5+cx;
            ty = y+j-0.5+cy;
            tz = cz+z+sz;
            if ~usercolor
                surf(tx,ty,tz,ccol);
            else
                h=surf(tx,ty,tz);
                set(h,'FaceColor',col);
            end
            patch(tx(2,:),ty(2,:),tz(2,:),col);
        end
    end
end
    
