/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
/* Define NDEBUG to turn off assertions, for faster execution */

// #define NDEBUG

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <values.h>
#include <time.h>
#include <bits/nan.h>
#include <assert.h>

#define BYTEINT signed char

/* Ranking: unfinished project for a linear ranking based selection */

// #define RANKING

/* Version 1 is the original fitness function from thesis */

#define VERSION2

#include "vector.h"
#include "pfit.h"

/* Makelineage: When defined, build a trace file of parents and descendants that you can 
 * use to trace back the ancestry of any individual.
 *  #define MAKELINEAGE 
 */


/* Enable a bunch of print statements for debugging */
// #define PRINTS


/* Define NODUMP to avoid saving .info structure files (still saves .m files) 
 * .info files not yet implemented for 3D 
 */
#define NODUMP

#define NOCROSSOVER 0

/* Random number seed = SRANDOM. Set to time(NULL) for random initialization, or any given integer for fixed 
 * sequence */

#define SRANDOM time(NULL) /* 0 */

#ifdef PARALLEL
#define USE_RAND48P
/* random library is not parallel- compatible, 
   but rand_48 seems to work well */
#endif

#ifdef USE_RAND48P
#define RANDINT lrand48P
#define RANDSEED srand48P
#else
#define RANDINT random
#define RANDSEED srandom
#endif

#define Kilo 1000
#define Mega (Kilo*Kilo)

#define POP_SIZE 2000

/* Gravitational constant. Usually 1.2 = 20% over measured gravitational force, 
 * for safety 
 */
#ifdef VERSION2
#define GRAVITY 5
#else
#define GRAVITY 1.2
#endif

#define ALPHA 2.0

#define WATCH_COUNT 10000

#ifdef VERSION2
#define TARGET_FITNESS 32767
#else
#define TARGET_FITNESS 4000
#endif

#define ADDFORCE_ABORTTIME 0

#define HEIGHT 1.1954 /* height of a brick, in width units */
#define WEIGHT 0.3958 /* weight of a unit mass */
#define LENGTH 8.0    /* width of a unit, in mm */

/* Modulus = positive reminder of x/y (assuming x y integer and y positive, fixes
 * the behavior of % which returns negative when x negative*/
#define MOD(x,y) (((x)%(y))+((x)>=0 ? 0 : (y)))

/* #define PRINTS */

/* size of predefined bricks array. no predef bricks in this experiment */
#define PREDEF_BRICK_NO 0


#define MAXBRICKS (150)
#define MAXBODIES (MAXBRICKS)
#define MAXJOINTS (MAXBODIES*(BODYJOINTS/2+1))

#define MAXADDFORCECOUNT 4*100*Kilo
#define COUNTLIMIT /* 150000 */ 2000*Mega

/* situation description */


/* base plate location */
#define PLATEX0 -2
#define PLATEX1 /* 32 */  2
#define PLATEY0 -2
#define PLATEY1 2

#ifdef UNDEF
#define TARGETX /* (-150) */ (-62) 
#define TARGETY (0)
#endif

#define MINY (-9)
#define MAXY (10)
#define MAXX (10)
#define MINX (-9)
#define MAXZ (15)
#define MINZ (0)

#ifndef VERSION2
#define TARGET_AREA ((-4+MAXY-1-MINY)*(-4+MAXX-1-MINX))
#endif

#define EMPTY (-2)
#define GROUND (-1)
#define TRANSIT (-3)
#define UNUSED (-4)
#define RESERVED (-5)

/* be careful with the following concepts. The names are incorrect and
they are used in precise ways at a couple of spots below... */

#define ISSOLID(thing) (((thing)>=0) || ((thing) == GROUND))
#define ISNBRICK(thing) (((thing) == EMPTY) || (thing)==GROUND)
#define ISBRICK(thing) (!ISNBRICK(thing))
#define ISEMPTY(thing) ((thing)==EMPTY)

#ifdef PARALLEL

int hasforked=0;
int haslocked ONEFOREACH;

void my_lock() {
	USEMINE
	if (hasforked && (haslocked MINE == 0)) m_lock();
	haslocked MINE ++;
#ifdef CHECKS
	if (haslocked MINE > 5) printf("%d:too many locks\n",m_get_myid());
#endif
}

void my_unlock() {
	USEMINE
	haslocked MINE --;
	if (hasforked && (haslocked MINE ==0)) m_unlock();
#ifdef CHECKS
	if (haslocked MINE < 0) printf("%d:lock underflow\n",m_get_myid());
#endif
}

#endif

/* addforce abortion mechanism */
int addforcecount ONEFOREACH;
unsigned long ringcount=0;


/* Gravity as a vector pointing straight down */
vector2d grav2d = { 0.0, -GRAVITY };

/* Maximum number of descendants a brick can have */
#define MAXSONS 5

typedef struct {
	/* attachment point of the brick wrt parent */
	int /*byte?*/ wherex,wherey,wherez; 
	 /* reference to node of brick attached here */
	short brickno; 
	 /* rotation 0 = no rotation, 1 = 90, 2 = 180, 3 = 270 */
	BYTEINT rot;   
} brick_descendant;

typedef struct {
	int /* byte? */ sizex,sizey; /* size of brick */
	brick_descendant joints[MAXSONS]; /* descendants of brick */
} brick;

char *str_descendant(brick_descendant *j) {
	static char s[200];
	sprintf(s,"%d %d %d %d",j->wherex,j->wherey,j->wherez,j->rot);
	return s;
}

/* allocate memory for brick structure */
brick bricks ONEFOREACH[MAXBRICKS];

/* allocate memory for description of physical substructure */


network2d bodiesx ONEFOREACH,bodiesy ONEFOREACH;

int brickcount ONEFOREACH;
REAL totalmass ONEFOREACH;
REAL fitnessvalue ONEFOREACH;



#define XSHIFT(j,js,bs,nbs) (((j)==0||(j)==3)?((js)-(nbs)):((bs)-(js)))
#define YSHIFT(j) (((j)==0||(j)==1)?-1:1)

/* GRID */

#define GRID BYTEINT
#define GMAXX (MAXX+1)
#define GMINX (MINX-1)
#define GMAXY (MAXY+1)
#define GMINY (MINY-1)
#define GMAXZ (MAXZ+1)
#define GMINZ (MINZ-1)


GRID grid ONEFOREACH [(GMAXX+1 - GMINX) * (GMAXY+1 - GMINY)*(GMAXZ+1-GMINZ)];
BYTEINT vgrid ONEFOREACH [(GMAXX+1-GMINX)] ;

#define GRIDOFF2(x,y) (((y)-GMINY)+(GMAXY-GMINY)*((x)-GMINX))
#define GRIDOFF3(x,y,z) ((z)-GMINZ)+(GMAXZ-GMINZ)*(GRIDOFF2(x,y))
#define GRIDOFF GRIDOFF3

int getgrid(int x, int y, int z) {
	USEMINE
	if (vgrid MINE [x-GMINX]==0) return EMPTY;
	else return grid MINE [GRIDOFF(x,y,z)];
}

int setgrid(int x, int y, int z, int value) {
	USEMINE
	register int j;

	if (vgrid MINE [x-GMINX]==0) {
		for (j=GRIDOFF(x,GMINY,GMINZ);j<GRIDOFF(x+1,GMINY,GMINZ);j++) 
			grid MINE [j]=EMPTY;
		vgrid MINE [x-GMINX]=1;
	}
	x = GRIDOFF(x,y,z);
	y=(GRID) grid MINE [x];
	grid MINE [x]=value;
	return y;
}


void initgrid() {
	USEMINE
	register int i,j;
	for (i=0;i< (sizeof (vgrid MINE)); i++)
		vgrid MINE [i]=0;
	for (i=PLATEX0;i<PLATEX1;i++)
		for (j=PLATEY0;j<PLATEY1;j++)
			setgrid(i,j,-1,GROUND);
	/*  reserve_spots(RESERVED); */
}

void makejoint(network2d *net, int body1, int body2, int sign, int jointsize, int jointwidth, REAL x, REAL y) {
	USEMINE
	joint *j;
	joint *joints=net->joints;
	body *bodies=net->bodies;


#ifdef PRINTS
	printf("makejoint %d %d %d %d %d %f %f\n",body1,body2,sign,jointsize,jointwidth,x,y);
#endif
	j=&joints[net->jointcount++];
	if (net->jointcount  == MAXJOINTS) {
		fprintf(stderr, "Error: Too many joints!\n");
		exit(1);
	}
	j->x.x = x;
	j->x.y = y;
	if (jointsize < 0) {
		fprintf(stderr, "Error: negative jointsize\n");
		exit(1);
	}
	switch(jointsize) {

	case 0: j->maxtorque = 0.0; break;
	case 1: j->maxtorque = 405.0; break;
	case 2: j->maxtorque = 1960.0; break;
	case 3: j->maxtorque = 3500.0; break;
	case 4: j->maxtorque = 6144.0 ; break;
	case 5: j->maxtorque = 11000.0 ; break;
	case 6: j->maxtorque = 13520.0; break;
	case 7: j->maxtorque = 14238.0; break;
	default:j->maxtorque = jointsize * 14238.0 / 7.0;
	}
	/* j->maxtorque = jointsize * TORQ_UNIT */;
	j->maxtorque *= jointwidth;
	j->n = net->jointcount;
	addjoint(&bodies [body1], sign, 
			((body2==GROUND) ? NULL : &bodies[body2]),j);
	if (body2 != GROUND)
		addjoint(&bodies [body2], -sign, &bodies [body1],j);
}


#define MAXSIZEX 20
#define MAXSIZEY 20

int newjoints3d(int gx, int gy, int gz, int bodyno, int sizex, int sizey, int sign) {
	USEMINE
	int i,j;
	static int lgrid[MAXSIZEX][MAXSIZEY];
	int what;
	int wx,wy,sx,sy,a,b;
	REAL jx,jy,jz;

	/* clear grid */
	for (i=0;i<sizex;i++)
		for (j=0;j<sizey;j++)
			lgrid[i][j]=0;

	/* scan */
	for (i=0;i<sizex;i++)
		for (j=0;j<sizey;j++)
			if ((!lgrid[i][j]) && 
					((what=getgrid(gx+i,gy+j,gz+sign)),ISSOLID(what))) {
				/* there is contact between bodyno and what at gx+i,gy+j,gz+sign. Study it */
				wx=gx+i;wy=gy+j;
				/* contact is assumed to be rectangular; measure its sizes, x and y */
				for(sx=1;(wx+sx < gx+sizex) && (what == getgrid(wx+sx,wy,gz+sign));sx++);
				for(sy=1;(wx+sy< gy+sizey) && (what == getgrid(wx,wy+sy,gz+sign));sy++);
				/* create joints */
				jx = ((REAL) wx + wx + sx) / 2.0;
				jy = ((REAL) wy + wy + sy) / 2.0;
				jz = ((REAL) gz + 0.5 + 0.5 * sign) * HEIGHT;
				makejoint(&bodiesx MINE, bodyno,what, sign,sx,sy,jx,jz);
				makejoint(&bodiesy MINE, bodyno,what, sign,sy,sx,jy,jz);
				/* cancel involved knobs */
				for(a=0;a<sx;a++)
					for (b=0;b<sy;b++)
						lgrid[i+a][j+b]=1;
				/* done. continue */
			}
	return 0;
}

int putbrickat(int nx, int ny, int gz,  int sx, int sy,int bodyno) {
	USEMINE
	int j,k;

	/* sx and sy must be nonnegative */

	/* prohibit to have bricks 'under the table' */
	/*
    if (gy < 0 && gx + size >= 0 )
    return 1;
	 */
	/* or out of bounds in general */
	if (ny+sy >= MAXY ||
			nx + sx >= MAXX ||
			gz >= MAXZ ||
			ny < MINY ||
			nx < MINX ||
			gz < MINZ)
		return 1;
	/* see if this brick fits and add it to the grid */
	for (j=0;j<sx;j++) 
		for (k=0;k<sy;k++)
			if (EMPTY != getgrid(nx+j,ny+k,gz))
				return 1;
	for (j=0;j<sx;j++) 
		for (k=0;k<sy;k++)
			setgrid(nx+j,ny+k,gz,bodyno);
	return 0;
}

void sonxy(int *sonx,int *sony,int gx,int gy, int rot, int wx, int wy) {
	switch(MOD(rot,4)) {
	case 0:
		*sonx=gx+wx;
		*sony=gy+wy;
		break;
	case 1:
		*sonx=gx-wy;
		*sony=gy+wx;
		break;
	case 2:
		*sonx=gx-wx;
		*sony=gy-wy;
		break;
	case 3:
		*sonx=gx-wy;
		*sony=gy-wx;
		break;
	}
}

void normbrick(int *gx, int *gy, int rot, int *sizex, int *sizey) {
	int nx, ny;
	int sx, sy;

	rot=MOD(rot,4);
	switch (rot) {
	case 0:
		nx=*gx;
		ny=*gy;
		sx=*sizex;
		sy=*sizey;
		break;
		
	case 1:
		nx=*gx-*sizey+1;
		ny=*gy;
		sx=*sizey;
		sy=*sizex;
		break;

	case 2:
		nx=*gx-*sizex+1;
		ny=*gy-*sizey+1;
		sx=*sizex;
		sy=*sizey;
		break;

	case 3:
		nx=*gx;
		ny=*gy-*sizex+1;
		sx=*sizey;
		sy=*sizex;
		break;
		
	default:
		fprintf(stderr,"normbrick: invalid argument rot=%d\n",rot);
		exit(1);
	}
	*gx=nx;
	*gy=ny;
	*sizex=sx;
	*sizey=sy;
}

int addbrick(int gx, int gy, int gz, int rot, int brickno) {
	USEMINE
	int j,k;
	int nx,ny,sx,sy;
	int sizex = bricks MINE [brickno].sizex;
	int sizey = bricks MINE [brickno].sizey;
	int bodyno;
	/* rot must be absolute already */
	/* first: rotate the brick */

	/* compute rotation */

	nx=gx;ny=gy;sx=sizex;sy=sizey;
	normbrick(&nx,&ny,rot,&sx,&sy);

#ifdef PRINTS
	printf("AB %d %d %d  %d(%d,%d) ",gx,gy,gz,brickno,
			bricks MINE [brickno].sizex,bricks MINE[brickno].sizey);
#endif


	/* see if this brick fits and add it to the grid */  
	bodyno = bodiesx MINE. bodycount;
	if (putbrickat(nx,ny,gz,sx,sy,bodyno))
		return 1; /* if it don't fit, you must acquit */

	/* create body */

	/* center of mass */

	bodiesx MINE .bodies[bodyno].x.x = 0.5 * sx + nx;
	bodiesy MINE .bodies[bodyno].x.x = 0.5 * sy + ny;

	bodiesx MINE .bodies[bodyno].x.y = 
		bodiesy MINE .bodies[bodyno].x.y = HEIGHT * (gz + 0.5);

	/* mass */
	bodiesx MINE .bodies[bodyno].mass = 
		bodiesy MINE .bodies[bodyno].mass = sx*sy;

	/* init */

	bodiesx MINE .bodies[bodyno].jointcount = 
		bodiesy MINE.bodies [bodyno]. jointcount = 0;

	bodiesx MINE.bodies[bodyno].n = 
		bodiesy MINE.bodies[bodyno].n = 
			brickno;

	totalmass MINE += sx * sy;

	if (bodyno== MAXBODIES) return 1;    /* failed: too many bodies */

	/* create joints */
	if (newjoints3d(nx,ny,gz,bodyno,sx,sy,-1) ||
			newjoints3d(nx,ny,gz,bodyno,sx,sy,1))
		return 1;                   /* failed */

	bodyno++;
	bodiesx MINE.bodycount =
		bodiesy MINE.bodycount=bodyno;
	{
		int sonx,sony,wx,wy;

		/* create linked bricks */
		for (j=0;j<MAXSONS;j++) {
			if ((k=bricks MINE[brickno].joints[j].brickno), ISBRICK(k)) {
				wx = bricks MINE [brickno].joints[j].wherex;
				wy = bricks MINE [brickno].joints[j].wherey;
#ifdef PRINTS
				printf("J %d(%d,%d) k=%d(%d,%d) ",j,bricks MINE[brickno].joints[j].wherex,
						bricks MINE[brickno].joints[j].wherey,k,
						bricks MINE[k].sizex,bricks MINE[k].sizey);
#endif
				sonxy(&sonx,&sony,gx,gy,rot,wx,wy);
				if (addbrick (sonx,sony,gz+bricks MINE [brickno].joints[j].wherez,
						rot+bricks MINE [brickno].joints[j].rot,
						bricks MINE [brickno].joints[j].brickno ))
					return 1;    /* failed */
			}
		}
	}
	return 0;
}  


int makemodel() {
	USEMINE
	int ret;

#ifdef PRINTS
	printf("MM ");
#endif
	initgrid();

	bodiesx MINE.jointcount =
		bodiesy MINE.jointcount = 0;
	bodiesx MINE.bodycount =
		bodiesy MINE.bodycount = 0;

	totalmass MINE= 0.0;
	ret = addbrick(0,0,0,0,0);
#ifdef PRINTS
	printf("ab000=%d ",ret);
#endif
#ifdef PREDEF_BRICKS 
	{
		int c,i,j;
		if (ret==0) {
			/* add predefined bricks */
			int c=brickcount MINE;
			reserve_spots(EMPTY); /* de-regulate previously reserved spots */
			for (i=0;i<PREDEF_BRICK_NO;i++) {
				bricks MINE [i+c] =  predef_bricks[i].brick;
				if (addbrick(predef_bricks[i].x,predef_bricks[i].y,i+c)) {
					fprintf(stderr,"Error adding predefs\n");
					/* exit(1); */ return 1;
				}
			}
			brickcount MINE = c +  PREDEF_BRICK_NO;
		}
	}
#endif
#ifdef PRINTS
	if (ret) printf ("MM Failed\n");
	else printf("\n");
#endif

	return ret;
}

void resetnet(network2d *bodies) {
	USEMINE
	int i;
	for (i=0; i<bodies->bodycount;i++)
		bodies->bodies[i].equilibrium = 1;
	for (i=0;i<bodies->jointcount;i++) {
		bodies->joints[i].torques = 0.0;
		bodies->joints[i].forces.x = bodies->joints [i].forces.y = 0.0;
	}
	addforcecount MINE =MAXADDFORCECOUNT;
}

int tooheavy2d(network2d *bodies) {
	USEMINE
	/* init joints and bodies */
	int i;

	resetnet(bodies);

	for (i=0;i<bodies->bodycount;i++) {
#ifdef PRINTS
		printf("addforce body %d of %d...\n",i,bodies->bodycount);
#endif
		if (.9999 > 
		addforce(&bodies->bodies[i],bodies->bodies[i].x,
				mul2d(bodies->bodies[i].mass,grav2d))) {
			if (addforcecount MINE== 0) {
				INCREMENT(ringcount);
#ifdef PRINTS
				printf("AAAddforce aborted\n");
#endif
			}
#ifdef PRINTS
			printf("Too heavy.\n");
#endif
			return 1;
		}
	}
	return 0;
}

int tooheavy() {
	USEMINE
	/* init joints and bodies */
	return 0;
	return tooheavy2d(&bodiesx MINE) || tooheavy2d(&bodiesy MINE);
}

#ifdef VERSION2
REAL fitness() {
	USEMINE
	int i,j,k;
	REAL count;
	int support;
	//vector2d loadvector;
	REAL ret;
	int myminX,myminY;
	int mymaxX,mymaxY;

	myminX=MINX;
	myminY=MINY;
	mymaxX=MAXX;
	mymaxY=MAXY;

	count=0;
	support=0;
	//loadvector=v2d(0.0,-1.0*(50/WEIGHT));

	REAL cumfit = 1;
	REAL fitHere;
	for (i=myminX;i<mymaxX;i++)
		for (j=myminY;j<mymaxY;j++) {
			for (k=MAXZ-1; k>=0 && getgrid(i,j,k) < 0; k--) { 
				//if (getgrid(i,j,k) >=0) 
				//	break;
				/*
				     if (k == (MAXZ-1)) 
						fitHere = 1.0;
					else
						fitHere = 0.5 * ((REAL)k) / ((REAL)(MAXZ-1));
				 */
				/*
					at.x = (REAL) i;
					at.y = (REAL) MAXZ-1;
					resetnet(&bodiesx MINE);
					if (.99 <= addforce(&bodiesx MINE.bodies[where],at,loadvector))
						support++;
					at.x=(REAL) j;
					resetnet(&bodiesy MINE);
					if (.99 <= addforce(&bodiesy MINE.bodies[where],at,loadvector))
						support++;
				 */
			}
			fitHere = ((REAL)k+2)/((REAL)(MAXZ+1));
			cumfit = cumfit * (fitHere/10 + 0.9);
#ifdef PRINTS
			//printf("%4d %10g %10g \n",k,fitHere,cumfit);
#endif
		}
	ret=cumfit;
	ret=ret*1000;
#ifdef PRINTS
	printf(" Fit=%f.\n",ret);
	//exit(0);
#endif
	//ret=exp(ret);
	return ret;
}

#else
// version 1 fitness
REAL fitness() {
	USEMINE
	int i,j;
	REAL count;
	int support,where;
	vector2d at,loadvector;
	REAL ret;
	REAL maxz;
	int myminX,myminY;
	int mymaxX,mymaxY;

	myminX=MINX+2;
	myminY=MINY+2;
	mymaxX=MAXX-3;
	mymaxY=MAXY-3;

	count=0;
	support=0;
	loadvector=v2d(0.0,-1.0*(50/WEIGHT));

	for (i=myminX;i<mymaxX;i++)
		for (j=myminY;j<mymaxY;j++)
			if (getgrid(i,j,MAXZ-1) >=0 )
				count += 1.0;
			else	if (getgrid(i,j,MAXZ-2) >= 0)
				count += 0.5;
			else	  if (getgrid(i,j,MAXZ-3) >=0)
				count += 0.25;

	if (count == 0.0)  {
		maxz = -10.0;

		for (i=0;i<brickcount MINE;i++)
			if (bodiesx MINE.bodies[i].x.y > maxz)
				maxz = bodiesx MINE.bodies[i].x.y;
		ret=1.0+maxz/(MAXZ-MINZ);
	}
	else if (count+4  < (REAL) TARGET_AREA) {
		ret=2.0+(REAL)count / TARGET_AREA;
	}
	else {
		for (i=myminX;i<mymaxX;i++)
			for(j=myminY;j<mymaxY;j++)
				if ((where=getgrid(i,j,MAXZ-1))>=0) {
					at.x = (REAL) i;
					at.y = (REAL) MAXZ-1;
					resetnet(&bodiesx MINE);

					if (.99 <= addforce(&bodiesx MINE.bodies[where],at,loadvector))
						support++;
					at.x=(REAL) j;
					resetnet(&bodiesy MINE);
					if (.99 <= addforce(&bodiesy MINE.bodies[where],at,loadvector))
						support++;
				}
		if (support + 8.0 <= 2*TARGET_AREA) {
			ret=3.0+(REAL)support/2.0/TARGET_AREA;
		}
		else {
			ret=4.0+2.0*TARGET_AREA/totalmass MINE;
		}
	}

#ifdef PRINTS
	printf(" Fit=%f.\n",ret);
#endif
	ret=exp(ret);
	return ret;
}

#endif

#define cmemcpy(p,q,a,b) memcpy(p,q,(a)*(b))

static struct {
	BYTEINT x,y;
} bricksizes[] = { {1,1}, {2,1}, {2,2}, {3,1}, {3,2}, {4,1}, {4,2},{6,1},{8,1},{10,1},{12,1}};
#define BRICKSIZES 11

typedef struct {
	brick *bricks;
	int brickcount;
	REAL fitness;
	REAL mass;
#ifdef PARALLEL
	unsigned long lock;
#endif
} individual;

int parent ONEFOREACH;

individual population[POP_SIZE];

int popcount = 0;

REAL maxfitness = -1e32;
REAL mass_maxfitness = NAN;

REAL scale_fitness(REAL f, REAL mass) {
	REAL raw = exp(f/maxfitness);
	REAL premium = 1.0 - (mass/(MAXBRICKS*8));
	if (premium > 0) 
		raw = raw + raw * premium / 10.0;
	return raw;
}

int select_on_fitness(int inverse /*REAL alpha*/ LOCK_PARAM) {
	REAL x,end,acc;
	int i;
	REAL f;
	
#ifdef PARALLEL
	int done;
#endif
	
#ifdef PARALLEL
	done=0;
	while (!done) {
#endif
		end = acc = 0.0;
		for (i=0;i<popcount;i++) { 
			/*    end += pow(population[i].fitness,alpha); */
			f = scale_fitness(population[i].fitness,population[i].mass);
			end += inverse ? (1.0 / f) : f;
		}
		x = (REAL) (RANDINT() % 1000000) / 1000000.0 * end;
		for (i=0;i<popcount;i++) {
			/* acc += pow(population[i].fitness,alpha); */
			f = scale_fitness(population[i].fitness, population[i].mass);
			acc += inverse ? (1.0 / f) : f;
			if (acc+NOTHING > x) break;
		}
#ifdef PARALLEL
			if (i<popcount) {
				LOCK
				if (to_write) {
					if (population[i].lock == 0) {
						population[i].lock = WRITE_LOCK;
						done=1;
					}
				}
				else {
					if ((population[i].lock & WRITE_LOCK)==0) {
						population[i].lock++;
						done=1;
					}
				}
				UNLOCK
			}
	}
#else
#ifdef CHECKS
	if (i<0 || i>= popcount)   
		fprintf(stderr,"out of bounds in select_on_fitness\n");
#endif
#endif
	return i;  
}

void validate_bricks(brick bricks[], int brickcount) {
	int i,j;
	int s; 
	for (i=0;i<brickcount;i++) {
		for (s=0;s<BRICKSIZES;s++) {
			if (bricksizes[s].x == bricks[i].sizex
					&& bricksizes[s].y == bricks[i].sizey)
				break;
		}
		assert(s<BRICKSIZES);
		for (j=0;j<MAXSONS;j++) {
			int desc = bricks[i].joints[j].brickno;
			if (desc!=EMPTY) {
				assert(desc<brickcount);
				assert(bricks[i].joints[j].wherex >= 0 && bricks[i].joints[j].wherex < bricks[i].sizex);
				assert(bricks[i].joints[j].wherey >= 0 && bricks[i].joints[j].wherey < bricks[i].sizey);
				assert((bricks[i].joints[j].wherez * bricks[i].joints[j].wherez) == 1);
				assert(bricks[i].joints[j].rot >= 0 && bricks[i].joints[j].rot < 4);
			}
		}
	}
}
		
int reorder_population(int i) {
	int o = i;
	individual temp = population[i];
	int flag=0;
	while (i>0 && population[i].fitness > population[i-1].fitness) {
		population[i]=population[i-1];
		i--;
		flag = 1;
	}
	while (i<(popcount-1) && population[i].fitness < population[i+1].fitness) {
		population[i]=population[i+1];
		i++;
		if (flag) {
			fprintf(stderr,"reorder_population: invalid population order at %d\n",i);
			exit(1);
		}
	}
	if (o != i)
		population[i] = temp;
	return i;
}

void print_bricks(brick bricks[], int brickcount) {
	int i;
	for (i=0;i<brickcount;i++) {
		int j;
		printf("brick %d size %d,%djoints ",
				i,
				bricks[i].sizex,
				bricks[i].sizey);
		
		for (j=0; j<MAXSONS; j++) {
			printf("%d(%d,%d,%d %d) ",bricks[i].joints[j].brickno,
					bricks[i].joints[j].wherex,
					bricks[i].joints[j].wherey,
					bricks[i].joints[j].wherez,
					bricks[i].joints[j].rot);
		}
		printf("\n");
	}
	printf("-----------\n");
}
int save_ind_at(int i) {
	
	USEMINE

#if (defined PARALLEL) && (defined CHECKS)
	if (population[i].lock != WRITE_LOCK) 
		fprintf(stderr,"warning: save at %d lock %x\n",i,population[i].lock);
#endif
	population[i].bricks = (brick *) calloc(brickcount MINE, 
			sizeof *(bricks MINE));
	if (population[i].bricks == NULL) {
		perror("OUT OF MEM save_ind_at");
		exit(1);
	}
#ifndef NDEBUG
	// asserts on
	validate_bricks(bricks MINE,brickcount MINE);
#endif
	cmemcpy(population[i].bricks, bricks MINE, brickcount MINE, sizeof *(bricks MINE));
	population[i].brickcount = brickcount MINE;
	population[i].fitness = fitnessvalue MINE;
	population[i].mass = totalmass MINE;
	
	
#ifdef RANKING
	i = reorder_population(i);
#endif
	
#ifdef PRINTS
	printf("SA %d bc=%d f=%f m=%f\n",
			i,brickcount MINE,fitnessvalue MINE, totalmass MINE);

	print_bricks(bricks MINE, brickcount MINE);
#endif
	return i;
}


/* forward declaration of mbricks */
void mbricks(char *name,brick bricks[],int brickno,double fit,network2d *bodies,network2d *bodiesy);

void erase_from_pop(int i) {
#ifdef UNDEF	
	if (!isnan(population[i].fitness)) {
		if (population[i].fitness == minfitness  
				|| population[i].fitness == maxfitness) {
			recalc_minmax_fitness(i);
		}
	}
#endif
	population[i].fitness = NAN;
	if (population[i].bricks != NULL) {
		free(population[i].bricks);
		population[i].bricks = NULL;
	}		
}

void update_population() {
	USEMINE
	int i,j;
	char s[50],t[50];
	static unsigned long count=0;
	unsigned long mycount;
#ifdef PARALLEL
	int locked;
#endif 
	mycount = INCREMENT(count);

#ifdef MAKELINEAGE
	if (lineage==NULL) lineage=fopen("lineage","w");
#endif
#ifdef SAVELINEAGE
	if (lineage==NULL) {
		lineage=fopen("linlist","r");
		fscanf(lineage,"%d %d %d",&nextsave,&i,&j);
		printf("next save at %d\n",nextsave);
	}
#endif


#ifdef PARALLEL
	if (popcount < POP_SIZE) {
		LOCK
		locked=1;
	}
	else locked=0;
#endif

	if (popcount == POP_SIZE) {
#ifdef PARALLEL
		if (locked) UNLOCK
#endif
		i=select_on_fitness(/*-ALPHA*/1 TO_WRITE);
		
		erase_from_pop(i);

	}
	else {
		i=popcount++;
#ifdef PARALLEL
		population[i].lock = WRITE_LOCK;
		population[i].fitness = 1.0;
		UNLOCK
#endif
	}

#ifdef MAKELINEAGE
	fprintf(lineage,"%d %d %d\n",count,parent,i);
	fflush(lineage);
#endif

	i = save_ind_at(i);
	
#ifdef PARALLEL
	test_then_add(&(population[i].lock),-WRITE_LOCK);
#endif

	/*
mbricks("test.m",bricks MINE,brickcount MINE,fitnessvalue MINE,
	&bodiesx MINE, &bodiesy MINE);

printf("Enter to continue: ");
scanf("%*s");
	 */
#ifdef SAVELINEAGE
	if (mycount >= nextsave) {
		printf("saving %d %d\n",mycount,nextsave);
		sprintf(t,"%5d",savecount++);
		for (i=0;t[i]!='\0';i++) if (t[i]==' ' ) t[i]='0';
		sprintf(s,"LIN%s.m",t);
		mbricks(s,bricks MINE,brickcount MINE,fitnessvalue MINE,
				bodiesx MINE, bodiesy MINE);
		fscanf(lineage,"%d %d %d",&nextsave,&j,&k);
		printf("next save at %d\n",nextsave);

	}
#endif

	count++;

	if (fitnessvalue MINE > maxfitness
	|| (fitnessvalue MINE == maxfitness && totalmass MINE < mass_maxfitness)) {
		LOCK
		if (fitnessvalue MINE > maxfitness) {
			maxfitness=fitnessvalue MINE;
			mass_maxfitness = totalmass MINE;
			sprintf(t,"%7d",(int) (fitnessvalue MINE*10000));
			for (j=0;t[j]!=0;j++) if (t[j]==' ') t[j]='0';
			WHOAM2
			fprintf(stderr,"MAX %s\n",t);
			sprintf(s,"MAX%s.m",t);
			mbricks(s,bricks MINE,brickcount MINE,fitnessvalue MINE,
					&bodiesx MINE, &bodiesy MINE);
			unlink("MAX.m");
			link(s,"MAX.m");
#ifndef NODUMP
			sprintf(s,"MAX%s.info",t);
			dumpfile(s,bricks MINE, brickcount MINE, bodies MINE, bodycount MINE, 
					joints MINE, jointcount MINE);
			unlink("MAX.info");
			link(s,"MAX.info");
#endif
#ifdef VBLOCKS
			sprintf(s,"MAX%s.vb",t);
			vb_bricks(s,bricks,brickcount,fitnessvalue);
			unlink("MAX.vb");
			link(s,"MAX.vb");
#endif
			UNLOCK
		}
	}
}


#ifdef PARALLEL
lock_ind(int i, int lock) {
	test_then_add(&(population[i].lock),lock);
}
#endif

void start_pop() {
	USEMINE
	int j;
	int i;
	for (i=0; i<POP_SIZE;i++) {
		population[i].bricks = NULL;
		population[i].fitness = NAN;
		population[i].brickcount = -1;
	}
	popcount = 0;
	bricks MINE[0].sizex = 2;
	bricks MINE[0].sizey = 2;
	for (j=0;j<MAXSONS;j++) {
		bricks MINE [0].joints[j].brickno= EMPTY;
	}
	brickcount MINE=1;
	if (makemodel() || tooheavy()) {
		fprintf(stderr,"Error in start pop 1\n");
		exit(1);
	}
	fitnessvalue MINE = fitness();
	update_population();
}


void randombricksize(int *x, int *y) {
	int i;
	i=(RANDINT() % BRICKSIZES);
	*x = bricksizes[i].x;
	*y = bricksizes[i].y;
}

void fixjoints (brick *b,int oldx,int oldy) {
	int i;
	if (b->sizex > oldx)
		for (i=0;i<MAXSONS;i++)
			b->joints[i].wherex += (RANDINT() % 1);
	if (b->sizey > oldy)
		for (i=0;i<MAXSONS;i++)
			b->joints[i].wherey += (RANDINT() % 1);

	for (i=0;i<MAXSONS;i++)
		b->joints[i].wherex = MIN(b->joints[i].wherex,b->sizex-1);

	for (i=0;i<MAXSONS;i++)
		b->joints[i].wherey = MIN(b->joints[i].wherey,b->sizey-1); 
}

void mutatebricksize(int *sx, int *sy) {
	int x;
	int delta;
	for (x=0;x<BRICKSIZES && 
	(*sx != bricksizes[x].x || *sy != bricksizes[x].y)
	;x++);
	if (x>=BRICKSIZES) {
		fprintf(stderr,"Illegal brick size %d %d\n",*sx,*sy); exit(1);
	}
	
	if (x==(BRICKSIZES-1))
		delta = (RANDINT() % 3) - 2; // -2 , -1 ,0
	else if (x == BRICKSIZES-2)
		delta = (RANDINT() % 4 ) - 2; // -2, -1 , 0, 1
	else if (x == 0)
		delta = (RANDINT() % 3) ; //  0, 1, 2
	else if (x == 1)
		delta = (RANDINT() % 4) - 1; // -1, 0 ,1, 2 
	else 
		delta = (RANDINT() % 5) - 2; // -2, -1, 0, 1, 2


	x += delta;

	assert(x>=0 && x<BRICKSIZES);
	*sx=bricksizes[x].x;
	*sy=bricksizes[x].y;
	/* joint positions must be updated after this (see fixjoints())*/
}

void randomjoint(brick_descendant *j, int sizex, int sizey) {
	assert(sizex>=0 && sizex<=16 && sizey>=0 && sizey <=2);
	j->wherex = RANDINT() % sizex;
	j->wherey = RANDINT() % sizey;
	j->wherez = (RANDINT()%2)*2-1;
	j->rot = RANDINT() % 4;
}

void mutatejoint(brick_descendant *j, int sizex, int sizey) {
//	brick_descendant pre = *j;

	int which;
	int options = 4; // 4 options: mutate x, y, z, rot. 
	if (sizex == 1) options--;
	if (sizey == 1) options--;
	which = RANDINT() % options;
	if (sizex > 1 && (options--) == 0) {
		if (j->wherex == 0) j->wherex = j->wherex + 1;
		else if (j->wherex == (sizex-1)) j->wherex = j->wherex - 1;
		else j->wherex = j->wherex + (RANDINT()%2)*2-1;
	}
	else if (sizey > 1 && (options--) == 0) {
		if (j->wherey == 0) j->wherey = j->wherey + 1;
		else if (j->wherey == (sizey-1)) j->wherey = j->wherey - 1;
		else j->wherey = j->wherey + (RANDINT()%2)*2-1;
	}
	else if ((options--) == 0) {
		/* mutate wz */
		j->wherez = -j->wherez;
	}
	else {
		j->rot +=  (RANDINT()%2*2)-1;
		j->rot = MOD(j->rot,4);
	}
	
	assert(j->wherex>=0 && j->wherex < sizex && j->wherey >=0 && j->wherey < sizey && (j->wherez*j->wherez)==1 && j->rot < 4 && j->rot >=0);
/*
	if (!good) {
		printf("%d %d %d %d\n",pre.wherex,pre.wherey,pre.wherez,pre.rot);
		printf("%d %d %d %d\n",j->wherex,j->wherey,j->wherez,j->rot);
		printf("%d %d\n",sizex,sizey);
		exit(1);
	}
 */	
}

int loadtree(int gx,int gy,int gz, int rot, brick *a, int acut, int aoff, brick *b, int boff) {
	USEMINE
	int j,k,joint,i;
	int nx,ny,sx,sy;

	if (brickcount MINE == (MAXBRICKS-PREDEF_BRICK_NO)) return 1; /* too big; failed */
	nx=gx;ny=gy;sx=a[0].sizex;sy=a[0].sizey;
	normbrick(&nx,&ny,rot,&sx,&sy);
	if (putbrickat(nx,ny,gz,sx,sy,brickcount MINE)) 
		return 1; /* doesn't fit */
	k=brickcount MINE++;
	bricks MINE [k].sizex = a[0].sizex;
	bricks MINE [k].sizey = a[0].sizey;
	for (j=0;j<MAXSONS;j++) {
		joint = a[0].joints[j].brickno;
		if (!ISBRICK(joint)) {
			assert(joint == EMPTY);
			memcpy(&bricks MINE[k].joints[j],&a[0].joints[j],sizeof bricks MINE[k].joints[j]);
			bricks MINE [k].joints[j].brickno = joint;
		}
		else /* if (joint >= 0) */ {
			int sonx,sony;
			assert(a[0].joints[j].wherex >= 0 && a[0].joints[j].wherex < a[0].sizex &&
					a[0].joints[j].wherey >=0 && a[0].joints[j].wherey < a[0].sizey &&
					(a[0].joints[j].wherez * a[0].joints[j].wherez) == 1 &&
					a[0].joints[j].rot >= 0 && a[0].joints[j].rot < 4);
					
			memcpy(&bricks MINE [k].joints[j],&a[0].joints[j],sizeof bricks MINE[k].joints[j]);
			bricks MINE [k].joints[j].brickno = brickcount MINE ;
			sonxy(&sonx,&sony,gx,gy,rot,a[0].joints[j].wherex,a[0].joints[j].wherey);
			if (joint == acut) 
				i = loadtree(sonx,sony,gz+a[0].joints[j].wherez,
						rot+a[0].joints[j].rot,
						b,UNUSED,boff,NULL,0);
			else
				i = loadtree(sonx,sony,gz+a[0].joints[j].wherez,
						rot+a[0].joints[j].rot,
						&a[joint-aoff],acut,joint,b,boff);
			if (i) {
				/* didn't fit, cut off rest */
				bricks MINE[k].joints[j].brickno=EMPTY;
			}
		}
	}
	return 0;
}

int mutate(brick *somebricks, int acount) {
	USEMINE
	int i,j,l;
	int oldx,oldy;
	brick abrick;
	LOCAL brick atree[MAXBRICKS];

#ifndef NDEBUG
	validate_bricks(somebricks, acount);
#endif
	cmemcpy(atree,somebricks,sizeof *atree,acount);
	initgrid();
	brickcount MINE=0;
#ifdef PRINTS
	printf("MUTATE s=%d ",acount);
	print_bricks(somebricks,acount);
#endif
	i= (RANDINT()%acount);
	if ((RANDINT()%2)==0) {
		// mutate brick size
#ifdef PRINTS
		printf("size mut %d,%d",atree[0].sizex,atree[0].sizey);
#endif
		oldx=atree[i].sizex;
		oldy=atree[i].sizey;
		mutatebricksize(&atree[i].sizex,&atree[i].sizey);
#ifdef PRINTS
		printf("->%d %d",atree[i].sizex,atree[i].sizey);
#endif
		fixjoints(&atree[i],oldx,oldy);
		if (loadtree(0,0,0,0,atree,UNUSED,0,NULL,0)) return 1;
	}
	else {
		int nsons = 0;
		int sons[MAXSONS];
		int which;
		int jj;
		int empty = -1;
		for (jj = 0; jj< MAXSONS; jj++) {
			if (atree[i].joints[jj].brickno != EMPTY) {
				sons[nsons] = jj;
				nsons ++;
			}
			else 
				empty = jj;
		}
		
		if (nsons == 0) 
			which = 1;
		else if (nsons == MAXSONS)
			which = (RANDINT()%nsons);
		else
			which = (RANDINT() % (nsons+1));
		
		if (which < nsons) {
			j = sons[which];
		}
		else { 
			j = empty;
			assert(j<MAXSONS);
		}
		
#ifdef PRINTS
		printf("joint mut br %d j %d (old %d)",i,j,atree[i].joints[j].brickno);
#endif
		if (atree[i].joints[j].brickno==EMPTY) {
			// add new descendant here
#ifdef PRINTS
			printf("NEW %d",j);
#endif
			if (acount+1 == (MAXBRICKS-PREDEF_BRICK_NO)) return 1; /* failed: can't add more */
			atree[i].joints[j].brickno=TRANSIT;
			for (l=0;l<MAXSONS;l++) {
				abrick.joints[l].brickno=EMPTY;
			}    
			randomjoint(&(atree[i].joints[j]),atree[i].sizex,atree[i].sizey);
			randombricksize(&abrick.sizex,&abrick.sizey);
#ifdef PRINTS
			printf("/%d,%d/%s/",abrick.sizex,abrick.sizey,str_descendant(&atree[i].joints[j]));
#endif
			if (loadtree(0,0,0,0,atree,TRANSIT,0,&abrick,0)) 
				return 1;
		}
		else {
			//int oldx=atree[k].sizex;
			//int oldy=atree[k].sizey;
			//mutatebricksize(&atree[k].sizex,&atree[k].sizey);
			//fixjoints(&atree[k],oldx,oldy);
			mutatejoint(&atree[i].joints[j], atree[i].sizex,atree[i].sizey);
#ifdef PRINTS
			printf("/%d,%d/%d,%d/",atree[i].sizex,atree[i].sizey,atree[i].joints[j].wherex,
					atree[i].joints[j].wherey);
#endif
			// here I allow mutation to kill the descendant
			loadtree(0,0,0,0,atree,UNUSED,0,NULL,0) ;
		}    
#ifdef PRINTS
		{
			int k = bricks MINE[i].joints[j].brickno;
			printf("now %d.%d=%d(%d %d)jw %d,%d ",i,j,
					k,
					bricks MINE[k].sizex,bricks MINE[k].sizey,
					bricks MINE[i].joints[j].wherex,bricks MINE[i].joints[j].wherey);
		}
#endif
		
	}
	if (makemodel()) {
		fprintf(stderr,"Error: Makemodel failed at mutate\n");
		/* exit(1); */ return 1; /* mutation failed */
	}
	if (tooheavy()) return 1;   /* collapsed! */
	fitnessvalue MINE= fitness();
	return 0;                /* mutation successful !*/
}

int crossover(brick a[], int acount, brick b[], int bcount) {
	USEMINE
	int i,j,x;

	i = RANDINT() % acount;
	j = RANDINT() % bcount;
	brickcount MINE=0;

#ifdef PRINTS
	printf("CROSS (%d)%d (%d)%d>",i,acount,j,bcount);
#endif
	initgrid();
	if (i==0) 
		x=loadtree(0,0,0,0,&b[j],UNUSED,j,NULL,0);
	else 
		x=loadtree(0,0,0,0,&a[0],i,0,&b[j],j);
	if (x) return 1;            /* didn't fit */
	if (makemodel()) return 1;  /* failed */
	if (tooheavy()) return 1;   /* collapsed! */
	fitnessvalue MINE = fitness();
	return 0;                /* crossover successful !*/
}

#define MMS(x,ma,mi,su) su += x; if(x > ma) ma = x; if(x < mi) mi = x;

void watch(count) {
	USEMINE
	int i,best = MININT ,worst = MININT;
	int minbricks,maxbricks,sumbricks;
	REAL minfit,maxfit,sumfit;
	REAL minmass,maxmass,summass;
	static int elapsed = 0;
	static int lastringcount = 1;
	static FILE *fp = NULL;

	WHOAM2
	fprintf(stderr,"watch %d\n",count);
	if (fp==NULL) fp=fopen("watch","w");
	minbricks = 100;
	maxbricks = -100;
	sumbricks = 0;
	minfit = minmass = 1e32;
	maxfit = maxmass = -1e32;
	sumfit = summass = 0.0;
	fprintf (fp,"%6d %d ",count,popcount);
	for (i=0;i<popcount;i++) {
#ifdef PARALLEL
		if ((population[i].lock & WRITE_LOCK)==0) {
#endif
			MMS(population[i].brickcount,maxbricks,minbricks,sumbricks);
			MMS(population[i].fitness,maxfit,minfit,sumfit);
			MMS(population[i].mass,maxmass,minmass,summass);
			if (population[i].fitness == maxfit) best=i;
			if (population[i].fitness == minfit) worst=i;
#ifdef PARALLEL
		}
#endif
	}
	fprintf (fp,"fi %f %f %f br %d %d %f ma %f %f %f ",
			maxfit,minfit,sumfit / popcount,
			maxbricks,minbricks, (REAL) sumbricks / popcount,
			maxmass,minmass,summass / popcount);
	initgrid();
	brickcount MINE=0;

	if(loadtree(0,0,0,0,population[best].bricks,UNUSED,0,NULL,0)
			|| makemodel() || tooheavy()) {
		fprintf(stderr,"Error: can't load best\n");
#ifndef NODUMP
		dumpfile("err",population[best].bricks, population[best].brickcount, 
				bodies MINE, bodycount MINE, 
				joints MINE, jointcount MINE);
#endif
		exit(1);
	}
	mbricks("BEST.m",bricks MINE,brickcount MINE,fitness(),
			&bodiesx MINE, &bodiesy MINE);

	initgrid();
	brickcount MINE=0;
	brickcount MINE=0;
	loadtree(0,0,0,0,population[worst].bricks,UNUSED,0,NULL,0);
	makemodel();
	tooheavy();
	mbricks("WORST.m",bricks MINE,brickcount MINE,fitness(),
			&bodiesx MINE, &bodiesy MINE);
	i=time(NULL);
	if (elapsed) fprintf(fp,"sec %d rc %lu ",i-elapsed,ringcount-lastringcount);
	if (elapsed) fprintf(stderr,"=%d secs.",i-elapsed);
	lastringcount=ringcount;
	elapsed=i;
	fprintf(fp,"\n");
	fflush(fp);

}




unsigned long compt =0, compf = 0, mutat = 0, mutaf = 0, crost = 0, crosf=0;

void evolve() {
	USEMINE
	int i,j;
#ifdef PARALLEL
	static unsigned long bigcount=0;
	static int rseed;
#endif
	int status;
	int count;



#ifdef PARALLEL
	count=INCREMENT(bigcount);
#ifdef PRINTS
	if(myid==0) rseed=SRANDOM;
	m_sync();
	printf("(%d) init evolve count=%d\n",myid,count);
	i=myid+rseed;
	RANDSEED(i);
	printf("(%d) init randseed=%d\n",myid,i);
#endif
#else
	count=0;
#endif

	while (maxfitness < TARGET_FITNESS && count < COUNTLIMIT) {
#if defined(PRINTS) && defined(PARALLEL)
		printf("(%d)evolve loop max=%f count=%d\n",myid,maxfitness,count);
#endif
		if ((!NOCROSSOVER) && RANDINT() % 2) {
			/* crossover */
			i=select_on_fitness(0 /*ALPHA*/ TO_READ);
			j=select_on_fitness(0 /*ALPHA*/ TO_READ);
#ifdef PRINTS
			printf("co %d %d ",i,j);
#endif
			parent MINE=i;
			INCREMENT(crost);
			status=crossover(
					population[i].bricks,
					population[i].brickcount-PREDEF_BRICK_NO,
					population[j].bricks,
					population[j].brickcount-PREDEF_BRICK_NO);
#ifdef PARALLEL
			test_then_add(&(population[i].lock),-1);
			test_then_add(&(population[j].lock),-1);
#endif
			if (status==0) 
				update_population();
			else {
				INCREMENT(crosf);
#ifdef PRINTS
				printf("co fail\n");
#endif
			}
		}
		else {
			/* mutate */

			/* plain mutation */
			i=select_on_fitness(0 /*ALPHA*/ TO_READ);
#ifdef PRINTS
			printf("mut %d ",i);
#endif
			parent MINE=i;
			INCREMENT(mutat);
			status=mutate(population[i].bricks,
					population[i].brickcount-PREDEF_BRICK_NO);

#ifdef PARALLEL
			test_then_add(&(population[i].lock),-1);
#endif
			if (status==0)
				update_population();
			else {
				INCREMENT(mutaf);
#ifdef PRINTS
				printf("mut fail\n");
#endif
			}      
		}
#ifdef PARALLEL
		count=INCREMENT(bigcount);

		if ((count % WATCH_COUNT) == 0) { LOCK watch(count); UNLOCK }; 
		/*

      if ((i=m_next()) >= WATCH_COUNT-PROCESSORS) {
      #ifdef CHECK
      if (haslocked[myid]) {
      fprintf(stderr,"panic: goint into sync with lock\n");
      }
      #endif
      m_sync();
      if (myid == 0) watch(count);
      m_sync();		
      }
		 */
#else
		count++;
		if ((count % WATCH_COUNT)==0) watch(count);
#endif
	}
}

void mtree(FILE *fp,int gx, int gy, int gz, int rot, int brickno, brick bricks[]) {
	brick *b;
	int j,k;
	int nx,ny,sx,sy;
	int sonx,sony;

	b=&bricks[brickno];
	nx=gx;ny=gy;sx=b->sizex;sy=b->sizey;
	normbrick(&nx,&ny,rot,&sx,&sy);
	fprintf(fp,"brick(%d,%d,%f,%d,%d,%f);\n",
			nx,ny,(REAL)gz*HEIGHT,
			sx,sy,HEIGHT);
	/*
  fprintf(fp,"text(%f,%f,'%d(%d)');\n",
	  (REAL) gx, (REAL)gy*HEIGHT + 0.5, brickno,bricks[brickno].size);
	 */  
	for (j=0;j<MAXSONS;j++) {
		k=b->joints[j].brickno;   
		if (k != EMPTY && k != GROUND) {
			sonxy(&sonx,&sony,gx,gy,rot,b->joints[j].wherex,b->joints[j].wherey);
			mtree(fp,
					sonx,sony,gz+b->joints[j].wherez,
					rot+b->joints[j].rot,
					k,bricks);
			/*
	fprintf(fp,"text(%f,%f,'%d');\n",
	(REAL) gx + ((j==0 || j==3) ? 0 : bricks[brickno].size - 2),
	(REAL) HEIGHT * (gy + ((j==0 || j==1) ? 0 : 1)),
	bricks[brickno].jointsize[j]);
			 */
		}

	}
}

void mbricks(char *name,brick bricks[],int brickno,double fit,network2d *bodies,network2d *bodiesy) {
	FILE *fp;
	body *b,*by;
	int i,j;
	REAL r1,r2;

	brickno++;

	fp=fopen(name,"w");
	fprintf(fp,"figure; hold on\n");
	mtree(fp,0,0,0,0,0,bricks);
	for (i=0;i<bodies->bodycount;i++) {
		b = &bodies->bodies [i];
		by = &bodiesy->bodies[i];
		fprintf(fp,"plot3(%f,%f,%f,'co');\n",b->x.x,by->x.x,b->x.y);
		for (j=0;j < b->jointcount;j++) {
			fprintf(fp,"plot3(%f, %f, %f,'y*');\n",
					b->joints[j]->x.x,
					by->joints[j]->x.x,
					b->joints[j]->x.y);    
			fprintf(fp,"x=plot3([%f %f],[%f %f],[%f %f]);\n",
					b->x.x,b->joints[j]->x.x,
					by->x.x,by->joints[j]->x.x,
					b->x.y,b->joints[j]->x.y);
			r1 = b->joints[j]->torques/b->joints[j]->maxtorque;
			r2 = by->joints[j]->torques/by->joints[j]->maxtorque;
			fprintf(fp,"set(x,'Color',[1 %f 0]);\n",MAX(r1,r2));
			fprintf(fp,"set(x,'LineWidth',%d);\n",
					(int) (7 * 0.5 *(b->joints[j]->maxtorque +by->joints[j]->maxtorque)/ 28000) + 1);
		}
	}
	fprintf(fp,"set(gca,'Title',text(0,0,'%f'))\n",fit);
	fclose(fp);
}


#ifdef VBLOCKS
/* VB_tree & VB_Bricks: v_blocks functions
 * this code hasn't been adapted to 3D */
void vb_tree(FILE *fp,int gx, int gy, int brickno) {
	brick *b;
	int j,k;

	b=&bricks[brickno];
	fprintf(fp,"%d     %d %d    %d %d %d       %d %d %d\n",
			1,
			1,b->size,
			0,(gy-4)*3,-(gx+b->size),
			0,0,0);
	for (j=0;j<MAXSONS;j++) {
		k=b->joints[j];    
		if (k != EMPTY && k != GROUND) {
			vb_tree(fp,
					(gx + XSHIFT(j,bricks[brickno].jointsize[j],
							bricks[brickno].size,bricks[k].size)),
							gy + YSHIFT(j),k);
		}
	}
}

void vb_bricks(char *name,brick bricks[],int brickno,double fit) {
	FILE *fp;
	body *b;
	int i,j;
	int gx,gy;
	char *code;

	brickno++;

	fp=fopen(name,"w");
	fprintf(fp,"2 	10 40 	-5 -13 -40	 0 	7 	0\n");
	vb_tree(fp,0,0,0);
	fclose(fp);
}


#endif

#ifndef NODUMP
/* Dump a text file summary of a structure. This code hasn't been adapted to 3D yet */
dumpfile(char *name, brick bricks[], int brickcount, body bodies[], int bodycount,
		joint joints[], int jointcount) {
	FILE *fp;
	int i,j;

	fp=fopen(name,"w");
	fprintf(fp,"%d\n\n",brickcount);
	for (i=0;i<brickcount;i++) {
		fprintf(fp,"%5d %d ",i,bricks[i].size);
		for (j=0;j<MAXSONS;j++) {
			fprintf(fp,"%d %d   ",bricks[i].joints[j],bricks[i].jointsize[j]);
		}
		fprintf(fp,"\n");
	}

	fprintf(fp,"\n\n---------bodies(%d)-------\n\n",bodycount);
	for (i=0;i<bodycount ;i++) {
		fprintf(fp,"%5d=%8x (%f,%f) eq %d mass %f ",i,&bodies[i],
				bodies[i].x.x,
				bodies[i].x.y,bodies[i].equilibrium,
				bodies[i].mass);
		for (j=0;j<bodies[i].jointcount;j++) 
			fprintf(fp,"%x(%x %d) ",bodies[i].joins[j],
					bodies[i].joints[j],bodies[i].jointsign[j]);
		fprintf(fp,"\n");
	}

	fprintf(fp,"\n\n-----------joints(%d)----------\n\n",jointcount);
	for (i=0;i<jointcount;i++) {
		fprintf(fp,"%5d=%8x (%f,%f) t=%f f=%f,%f, mt=%f\n",
				i,
				&joints[i],
				joints[i].x.x,joints[i].x.y,
				joints[i].torques,joints[i].forces.x,joints[i].forces.y,
				joints[i].maxtorque);
	}

	fclose(fp);
}

#endif

int main(int argc,char **argv) {
	int i;
#ifdef PARALLEL
	int myid=0;
#endif

#ifdef PARALLEL
	for (i=0;i<PROCESSORS;i++) {
		haslocked[i]=0;
	}
#endif

	/* allocate memory for loading and testing one model (per CPU) */
#ifdef PARALLEL
	for (i=0;i<PROCESSORS;i++)    {
		bodiesx[i].joints=malloc(MAXJOINTS * (sizeof *bodiesx[i].joints));
		bodiesx[i].bodies=malloc(MAXBODIES * (sizeof *bodiesx[i].bodies));
		bodiesy[i].joints=malloc(MAXJOINTS * (sizeof *bodiesy[i].joints));
		bodiesy[i].bodies=malloc(MAXBODIES * (sizeof *bodiesy[i].bodies));
	}
#else
	bodiesx.joints=malloc(MAXJOINTS * (sizeof *bodiesx.joints));
	bodiesx .bodies=malloc(MAXBODIES * (sizeof *bodiesx.bodies));
	bodiesy.joints=malloc(MAXJOINTS * (sizeof *bodiesy.joints));
	bodiesy.bodies=malloc(MAXBODIES * (sizeof *bodiesy.bodies));

#endif

	if (argc==1) {
		/* no arguments, initialize with random */
		start_pop();
		maxfitness=-1e32;
		mass_maxfitness = -1e32;
	}
	else {
#ifndef NODUMP
		printf("loading %s...\n",argv[1]);
		FILE *in=fopen(argv[1],"r");
		fscanf(in,"%d",&brickcount);
		for (i=0;i<brickcount MINE;i++) {
			fscanf(in,"%d %d",&z,&x);
			printf("%d %d ",z,x);
			bricks MINE[i].size = x;
			for (j=0;j<MAXSONS;j++) {
				fscanf(in,"%d %d",&x,&y);
				bricks MINE [i].joints[j] = x;
				bricks MINE [i].jointsize[j] = y;
				printf("%d(%d) ",x,y);
			}
			printf("\n");
		}
		if (makemodel()) {
			fprintf(stderr,"first makemodel failed!\n");
			mbricks("fail.m",bricks MINE,brickcount MINE,0.0, bodies MINE, bodycount MINE, joints MINE,
					jointcount MINE);
			KILL
			exit(1);
		}
		if (tooheavy()) {
			fprintf(stderr,"addforcecount = %d\n",addforcecount MINE );
			fprintf(stderr,"first tooheavy failed! dumping fail.m\n");
			mbricks("fail.m",bricks MINE,brickcount MINE,0.0, bodiesx,bodiesy);
			KILL
			exit(1);
		}
		fitnessvalue MINE = minfitness = fitness();
		maxfitness = -1e32;
		mass_maxfitness = -1e32;
		mbricks("START.m",bricks MINE,brickcount MINE,
				fitnessvalue MINE,bodiesx MINE, bodiesy MINE);
		/*      dumpfile("START.info",bricks MINE,brickcount MINE,
	    bodies MINE,bodycount MINE,joints MINE,jointcount MINE); */
		popcount=0;
		update_population();
#endif
	}
	i=SRANDOM;
	fprintf(stderr,"fit (%s) starts with seed %d\n",argv[0],i);
	RANDSEED(i);
#ifdef PARALLEL
	m_set_procs(PROCESSORS);
	hasforked=1;
	m_fork(evolve);
#else
	evolve();
#endif
	return 0;
}

/*  
xymain() {
  int x,y,z;
  initgrid();
  while (1) {
    scanf("%d %d",&x,&y);
    printf("%d %d = %d\n",x,y,getgrid(x,y));
    printf("new value-> ");
    scanf("%d",&z);
    printf("%d %d (%d) = %d == ",x,y,setgrid(x,y,z),z);
    printf("%d \n",getgrid(x,y));
  }
}

 */
